/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Class that maps an object to a Map.
 * <p>
 * When creating a <tt>MapMap</tt>, the user can choose to make
 * it synchronized or not. If the <tt>MapMap</tt> is created in
 * a synchronized mode, it will internally encapuslate a <tt>Hashtable</tt>
 * that contains name-value pair where the values are other 
 * <tt>Hashtable</tt> instances. In a non-synchronized mode, the internal 
 * map object will be a <tt>HashMap</tt> containing other
 * <tt>HashMap</t>s. * 
 * @author TorgilZ
 *
 */
public class MapMap<K, L, V> implements Serializable {

	private static final int	DEFAULT_MAP_INITIAL_CAPACITY = 16;
	
	private Map<K, Map<L, V>>  m_mapmap;
	private boolean            m_isSynchronized;
	private int                m_initialMapCapacity;
	
	/**
	 * Creates a non-synchronized <tt>MapMap</tt>.
	 *
	 */
	public MapMap() {
		this(DEFAULT_MAP_INITIAL_CAPACITY, false);
	}
	
	/**
     * Creates a synchronized or non-synchronized <tt>MapMap</tt>.
     * 
     * @param isSynchronized
     */
	public MapMap(boolean isSynchronized) {
		this(DEFAULT_MAP_INITIAL_CAPACITY, isSynchronized);
	}
	
	/**
     * Creates a non-synchronized <tt>MapMap</tt> with the specified internal
     * capacity of the internal <tt>Maps</tt>.
     * 
     * @param initialMapCapacity
     */
	public MapMap(int initialMapCapacity) {
		this(initialMapCapacity, false);
	}

	/**
     * Creates a synchronized or non-synchronized MapMap with the specified
     * internal capacity of the internal <tt>Maps</tt>.
     * 
     * @param initialMapCapacity
     * @param isSynchronized
     */
	public MapMap(int initialMapCapacity, boolean isSynchronized) {
		if(isSynchronized) {
			m_mapmap = new Hashtable<K, Map<L, V>>(initialMapCapacity);
        }
		else {
			m_mapmap = new HashMap<K, Map<L, V>>(initialMapCapacity);
        }
		m_isSynchronized = isSynchronized;
		m_initialMapCapacity = initialMapCapacity;
	}
	
	/**
     * Inserts the specified value into the MapMap with the specified keys.
     * 
     * @param key1
     * @param key2
     * @param value
     * @return The object that was previously mapped to the specified keys.
     */
	public V put(K key1, L key2, V value) {
		Map<L, V> map = m_mapmap.get(key1);
        if (map == null) {
            map = createMap();
            m_mapmap.put(key1, map);
        }
        return map.put(key2, value);
	}
	
	/**
     * Returns the object that is mapped to the specified keys.
     * 
     * @param key1
     * @param key2
     */
	public V get(K key1, L key2) {
		Map<L, V> m = m_mapmap.get(key1);
		return (m == null) ? null : m.get(key2);
	}
	
	/**
     * Removes the object that is mapped to the specified keys.
     * 
     * @param key1
     * @param key2
     * @return the object that was removed.
     */
	public V remove(K key1, L key2) {
		Map<L, V> m = m_mapmap.get(key1);
		return (m == null) ? null : m.remove(key2);
	}
    
    /**
     * Removes the <tt>Map</tt> that is mapped to the specified key.
     * 
     * @param key
     * @return the <tt>Map</tt> that was removed.
     */
    public Map<L, V> remove(K key) {
        return m_mapmap.remove(key);
    }
	
	/**
     * Returns the Map that is mapped to the specified key.
     * 
     * @param key1
     */
	public Map<L, V> get(K key1) {
		return m_mapmap.get(key1);
	}
	
	/**
     * Inserts the specified Map into the MapMap with the specified key.
     * 
     * @param key1
     * @param map
     * @return The Map that was previously mapped to the specified key.
     */
	public Map<L, V> put(K key1, Map<L, V> map) {
		return m_mapmap.put(key1, map);
	}
	
	/**
	 * Removes everything from this MapMap.
	 *
	 */
	public void clear() {
		for (Iterator<K> it1 = m_mapmap.keySet().iterator(); it1.hasNext();) {
            Map<L, V> map = m_mapmap.get(it1.next());
            map.clear();
            it1.remove();
        }
	}
	
	public Set<K> keySet() {
		return m_mapmap.keySet();
	}
	
	public Set entrySet() {
		return m_mapmap.entrySet();
	}
	
	public int size() {
		return m_mapmap.size();
	}
	
	public boolean contains(Object key1) {
		return m_mapmap.containsKey(key1);
	}
	
	public boolean contains(K key1, L key2) {
		Map<L, V> map = get(key1);
		return (map != null) ? map.containsKey(key2) : false;
	}
	
	
	private Map<L, V> createMap() {
		Map<L, V> map = m_isSynchronized ? (Map<L, V>)new Hashtable<L, V>(m_initialMapCapacity) :
				(Map<L, V>)new HashMap<L, V>(m_initialMapCapacity);
		return map;
	}
	
}
