/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.collections;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**`
 * Holds key-value pairs of strings. The collection is thread safe.
 * All keys in the collection need to have size > 1, however the value
 * for a key can be null or an empty string.
 * 
 * @author EmanuelN
 * @author Torgil
 */
public class StringKeyValuePairs {
    
    private Map<String,String> m_data;

    /**
     * Create an empty set of key-value pairs.
     */
    public StringKeyValuePairs() {
        super();
        m_data = Collections.synchronizedMap( new HashMap<String,String>() );
    }
    
    /**
     * Creates a copy of the original <tt>StringKeyValuePairs</tt>.
     * @param original  the <tt>StringKeyValuePairs</tt> instance to
     *                  create a copy of.
     * @throws IllegalArgumentException if <tt>original</tt> is null.                  
     */
    public StringKeyValuePairs(StringKeyValuePairs original) {
        if( original == null ) {
            throw new IllegalArgumentException("original can't be null.");
        }
        synchronized( original ) {
            HashMap<String, String> copy = 
                new HashMap<String, String>(original.m_data.size());
            for( String key : original.m_data.keySet() ) {
                copy.put(key, original.m_data.get(key) );
            }
            m_data = Collections.synchronizedMap(copy);
        }
    }
	
	public StringKeyValuePairs(String[] keys, String[] values) {
		if ( keys == null || values == null )
			throw new IllegalArgumentException("Both keys and values have to be non-null");
		if ( keys.length != values.length)
			throw new IllegalArgumentException("Both the keys and values arrays have to be of the same size");
		for ( int i = 0, N = keys.length; i < N; ++i ) {
			putString(keys[i], values[i]);
		}
	}
    
    /**
     * Returns the number of key-value pairs in this <code>StringKeyValuePairs</code>.
     */
    public int size() {
        return m_data.size();
    }
    
    /**
     * Returns the value for the key sKey. If there is no such key, null is returned.
     * Convenience method which forwards the call to getString.
     * 
     * @param sKey  the key for which to get the value.
     * @return The value of sKey, or null if no value exists for that key.
     * @see #get(String, String)
     * @see #getString(String)
     */
    public String get(String sKey) {
		return getString(sKey);
    }

	/**
     * Returns the value for the key sKey. If there is no such key, null is returned.
     * This is the main entry point for the "get" methods. The other ones will forward
     * the calls to this one.
     * 
     * @param sKey  the key for which to get the value.
     * @return The value of sKey, or null if no value exists for that key.
     * 
     */
	public String getString(String sKey) {
		if ( sKey == null || sKey.length() == 0 )
			return null;
		return m_data.get(sKey);
	}
    
    /**
     * Returns the value for the key <tt>sKey</tt>. If there is no such key,
     * the default value as specified by <tt>sDefault</tt> is returned.
     * @param sKey  the key for which to get the value.
     * @param sDefault  the default value to return if the specified key
     *                  doesn't exist in this <tt>StringKeyValuePairs<tt>.
     * @return  the String mapped to <tt>sKey</tt>, or <tt>sDefault</tt> if
     *          <tt>sKey</tt> doesn't exist.
     * @see #get(String)          
     */
    public String get(String sKey, String sDefault) {
        String val = getString(sKey);
        return (val != null) ? val : sDefault;
    }
		    
    /**
     * Convenience method that forwards the call to 
     * {@link #putString(String, String)}.
     * @param sKey
     * @param sValue
     */
    public void put(String sKey, String sValue) {
        putString(sKey, sValue);
    }
    
    /**
     * Adds the value sValue for the key sKey.
     * This is the main "put" method, all other "put" calls should be forwarded to this one.
     * 
     * @param sKey The key to set the value for. If sKey is null nothing happens.
     * @param sValue The value of the specified key.
     *               If sValue is null, the key will be removed and all "get" methods
     *               for sKey will return null.
     */
    public void putString(String sKey, String sValue) {
        if ( sKey == null || sKey.length() == 0)
            return;
		if ( sValue == null ) {
			remove(sKey);
			return;
		}
        m_data.put(sKey, sValue);
    }
    
    /**
     * Adds the value <code>iValue</code> for the key <code>sKey</code>.
     * <code>iValue</code> will be converted to a String through 
     * <code>String.valueOf(iValue)</code>.
     * @param sKey The key to set the value for. If sKey is null nothing happens.
     * @param iValue	The value of the specified key.
     */
    public void putInt(String sKey, int iValue) {
		putString(sKey, String.valueOf(iValue));
    }
    
    /**
     * Retrieves the <tt>int</tt> value that has been stored with the
     * specified key. The stored <tt>String</tt> value will be converted
     * to an <tt>int</tt> through the <tt>Integer.parseInt(String)</tt>
     * method.
     * @param sKey  the key to the mapped value
     * @return  the <tt>int</tt> value stored for the specified key.
     * @throws NumberFormatException    if the value stored for the
     *          specified key cannot be parsed to an <tt>int</tt>, or
     *          if nothing is mapped to the key.
     * @see #getInt(String, int)          
     */
    public int getInt(String sKey) throws NumberFormatException {
        return Integer.parseInt(getString(sKey));
    }
    
    /**
     * Retrieves the <tt>int</tt> value that is stored for the specified
     * key. The stored <tt>String</tt> value will be converted
     * to an <tt>int</tt> through the <tt>Integer.parseInt(String)</tt>
     * method. If the stored value cannot be converted to an <tt>int</tt>,
     * or if nothing is mapped to the key, <tt>defValue</tt> will be 
     * returned.
     * @param sKey  the key to the mapped value
     * @param defValue  the default value that will be used if the stored
     *                  value cannot be converted to an <tt>int</tt>.
     * @return  the <tt>int</tt> value stored for the specified key; 
     *          or <tt>defVal</tt> if there is nothing mapped to the specified
     *          key or if the mapped value cannot be converted to an 
     *          <tt>int</tt>.
     * @see #getInt(String)          
     */
    public int getInt(String sKey, int defValue) {
        try {
            return Integer.parseInt(getString(sKey));
        } 
        catch( NumberFormatException e ) {
            return defValue;
        }
    }
    
    /**
     * Adds the value <code>lValue</code> for the key <code>sKey</code>.
     * <code>lValue</code> will be converted to a String through 
     * <code>String.valueOf(lValue)</code>.
     * @param sKey The key to set the value for. If sKey is null nothing happens.
     * @param lValue	The value of the specified key.
     */
    public void putLong(String sKey, long lValue) {
		putString(sKey, String.valueOf(lValue));
    }

    /**
     * Retrieves the <tt>long</tt> value that has been stored with the
     * specified key. The stored <tt>String</tt> value will be converted
     * to an <tt>int</tt> through the <tt>Long.parseLong(String)</tt>
     * method.
     * @param sKey  the key to the mapped value
     * @return  the <tt>long</tt> value stored for the specified key.
     * @throws NumberFormatException    if the value stored for the
     *          specified key cannot be parsed to an <tt>long</tt>, or
     *          if nothing is mapped to the key.
     * @see #getLong(String, long)        
     */
    public long getLong(String sKey) throws NumberFormatException {
        return Long.parseLong(getString(sKey));
    }
    
    /**
     * Retrieves the <tt>long</tt> value that is stored for the specified
     * key. The stored <tt>String</tt> value will be converted
     * to an <tt>long</tt> through the <tt>Integer.parseInt(String)</tt>
     * method. If the stored value cannot be converted to a <tt>long</tt>,
     * or if nothing is mapped to the key, <tt>defValue</tt> will be 
     * returned.
     * @param sKey  the key to the mapped value
     * @param defValue  the default value that will be used if the stored
     *                  value cannot be converted to an <tt>long</tt>.
     * @return  the <tt>long</tt> value stored for the specified key; 
     *          or <tt>defVal</tt> if there is nothing mapped to the specified
     *          key or if the mapped value cannot be converted to a 
     *          <tt>long</tt>.
     * @see #getLong(String)          
     */
    public long getLong(String sKey, long defValue) {
        try {
            return Long.parseLong(getString(sKey));
        } 
        catch( NumberFormatException e ) {
            return defValue;
        }
    }
        
    // TODO: Think about the following method. By adding "1" for true and
    // "0" for false we are not following the String.valueOf pattern.
    /**
     * Adds the value <code>bValue</code> for the key <code>sKey</code>.
     * If <code>bValue</code> is <code>true</code> the <code>String</code> "1"
     * will be added; if <code>bValue</code> is <code>false</code> the <code>String</code>
     * "0" will be added.
     * @param sKey The key to set the value for. If sKey is null nothing happens.
     * @param bValue	The value of the specified key.
     */
    public void putBoolean(String sKey, boolean bValue) {
		putString(sKey, bValue ? "1" : "0");
    }
    
    /**
     * Retrieves the <tt>boolean</tt> value that has been mapped to the 
     * specified key. This method will return <tt>true</tt> if and only
     * if the value mapped to the key is "1".
     * @param sKey  the key of the mapped value.
     * @return  <tt>true</tt> if the mapped String value is "1";
     *          otherwise <tt>false</tt>.
     */
    public boolean getBoolean(String sKey) {
        String s = getString(sKey);
        return "1".equals(s);
    }
    
    /**
     * Retrieves the <tt>boolean</tt> value that has been mapped to the
     * specified key. If nothing has been mapped to the key, this method will
     * return <tt>defValue</tt>.
     * 
     * @param sKey
     *            the key of the mapped value.
     * @param defValue
     *            the default value to return if there is nothing mapped to the
     *            specified key.
     */
    public boolean getBoolean(String sKey, boolean defValue) {
        String s = getString(sKey);
        return ( s == null || s.length() == 0 ) ?
            defValue : s.equals("1");
    }

    /**
     * Adds the Object <code>value</code> for the key <code>sKey</code>. If
     * <tt>value</tt> is null, the inserted value will be <tt>null</tt>,
     * otherwise the inserted value will be <tt>value.toString()</tt>.
     * 
     * @param sKey
     *            The key to set the value for. If sKey is null nothing happens.
     * @param value
     *            The value of the specified key.
     */
    public void putObject(String sKey, Object value) {
		putString(sKey, (value != null) ? value.toString() : null);
    }
	
	public void putAll(StringKeyValuePairs keyValues) {
		for( String sKey : keyValues.getKeys() ) {
			putString(sKey, keyValues.getString(sKey));
		}
	}
    
    /**
     * Removes the key-value for the specified key.
     * 
     * @param sKey
     *            The key-value that should be removed.
     */
    public void remove(String sKey) {
        m_data.remove(sKey);
    }

     /**
         * Returns a Set<String> of all the keys defined by this object. The
         * returned Set contains a snapshot of all the keys at the time of the
         * call so care should be taken that the collection is not changed while
         * the keys in the set are being used.
         * 
         * @return Set<String> of keys.
         */
    public Set<String> getKeys() {
        return new HashSet<String>(m_data.keySet());
    }
    
	/**
	 * Put the key-value pair defind by the string. The string has the format
	 * "<key>=<Value>". The value can contain the separator "=".
	 * 
	 * @param sKeyValuePair String defining the key-value pair.
	 */
	public void putKeyValuePair(String sKeyValuePair) {
	    putKeyValuePair(sKeyValuePair, "=");
	}
	
    /**
     * Put the key-value pair defined by the string. The string has the format "<Key><Separator><Value>".
     * The Value can contain the separator.
     * 
     * @param sKeyValuePair
     *            String defining the key-value pair.
     * @param sKeyValueSeparator
     *            The separator used to separate the key from the value.
     */
	public void putKeyValuePair(String sKeyValuePair, String sKeyValueSeparator) {
	    if ( sKeyValuePair == null || sKeyValueSeparator == null)
	        return;
	    
	    int index = sKeyValuePair.indexOf(sKeyValueSeparator);
	    if ( index > 0 ) {
	        String sKey = sKeyValuePair.substring(0, index);
	        String sValue = sKeyValuePair.substring(index+1);
	        putString(sKey, sValue);
	    }	    
	}
	
	/**
     * Returns <tt>true</tt> if this object contains a value for the specified
     * key, otherwise <tt>false</tt> is returned.
     * 
     * @param key
     *            The key to query.
     */
	public boolean containsKey(String key) {
		return m_data.containsKey(key);
	}
	
    /**
     * Returns the key-values as a string using the format <key1>=<value1>;<key2>=<value2>...<keyN>=<valueN>.
     * This method should be used for human readable debugging and not as a
     * means of getting all the values. Use the getKeys() method for that
     * purpose.
     * 
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder(m_data.size()*10);
        for( String sKey : getKeys() ) {
            sb.append(sKey);
            sb.append("=");
            sb.append(getString(sKey));
            sb.append(";");
        }
        if ( sb.length() > 0 )
            sb.deleteCharAt(sb.length()-1);

        return sb.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if( obj == this ) {
            return true;
        }
        if( obj instanceof StringKeyValuePairs ) {
            // We can use the AbstractMap#equals to compare the objects
            // (refer to the AbstractMap javadocs for details):
            return m_data.equals( ((StringKeyValuePairs)obj).m_data );
        }
        return false;
    }

    @Override
    public int hashCode() {
        // We can use the AbstractMap#hashCode to compare the objects
        // (refer to the AbstractMap javadocs for details):
        return m_data.hashCode();
    }
}
