/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml.sax;

import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Class that provides default implementation of the functionality defined by
 * the <tt>KongaSaxParser</tt> interface.
 * <p>
 * A subclass of this class will typically need to implement at least the 
 * following methods:
 * <UL>
 *  <LI><tt>startElement</tt>: To ensure correct behavior, the subclass 
 *      implementation should begin by calling 
 *      <tt>super.startElement</tt>;</LI>
 *  <LI><tt>endElement</tt>: To ensure currect behavior, the subclass
 *      implementation should end by calling <tt>super.endElement</tt>.</LI>
 *      
 * @author Torgil
 *
 */
public class AbstractKongaSaxParser extends DefaultHandler 
                                implements IKongaSaxParser {

    private String                      m_currentElementValue = "";
    private Stack<ElementInformation>   m_elements = new Stack<ElementInformation>();
    
    /**
     * 
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getCurrentElementValue()
     */
    public String getCurrentElementValue() {
        return m_currentElementValue.trim();
    }

    /**
     * 
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getCurrentLocalName()
     */
    public String getCurrentLocalName() {
        return m_elements.empty() ? null : m_elements.peek().LocalName;
    }

    /**
     * 
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getCurrentURI()
     */
    public String getCurrentURI() {
        return m_elements.empty() ? null : m_elements.peek().URI;
    }

    /**
     * 
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getCurrentQName()
     */
    public String getCurrentQName() {
        return m_elements.empty() ? null : m_elements.peek().QName;
    }

    /**
     * 
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getCurrentAttributes()
     */
    public Attributes getCurrentAttributes() {
        return m_elements.empty() ? null : m_elements.peek().Attributes;
    }

    /**
     * Default implementation returns null.
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getStartElementHandler(java.lang.String, java.lang.String, java.lang.String)
     */
    public IStartElementHandler getStartElementHandler(String uri,
                                                      String localName, 
                                                      String qName) {
        return null;
    }

    /**
     * Default implementation returns null.
     * @see org.jitterbit.util.xml.sax.IKongaSaxParser#getEndElementHandler(java.lang.String, java.lang.String, java.lang.String)
     */
    public IEndElementHandler getEndElementHandler(String uri, 
                                                  String localName,
                                                  String qName) {
        return null;
    }

    @Override
    public void characters(char[] ch, int start, int length) throws SAXException {
        m_currentElementValue += new String(ch, start, length);
    }

    /**
     * A subclass implementation of this method should end by calling
     * <tt>super.endElement</tt> to ensure correct behavior of this class.
     * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
     */
    @Override
    public void endElement(String uri, String localName, String qName) throws SAXException {
        m_elements.pop();
        m_currentElementValue = "";
    }

    /**
     * A subclass implementation of this method should begin by calling 
     * <tt>super.startElement</tt> to ensure correct behavior of this class.
     * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
     */
    @Override
    public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
        ElementInformation info = new ElementInformation();
        info.URI = uri;
        info.LocalName = localName;
        info.QName = qName;
        info.Attributes = attributes;
        m_elements.push(info);
    }

    
    /**
     * Structure for holding element information. 
     * @author Torgil
     */
    private static class ElementInformation {
        public String       URI;
        public String       LocalName;
        public String       QName;
        public Attributes   Attributes;
    }
    
}
